@@ -1,12 +1,12 @@
 use anyhow::Result;
 
-use crate::book::{Book, Loader};
-use crate::html_convertor::html_lines;
+use crate::book::{Book, Line, Loader};
+use crate::html_convertor::{html_content, HtmlContent};
 
 pub(crate) struct HtmlLoader {}
 
 pub(crate) struct HtmlBook {
-	lines: Vec<String>,
+	content: HtmlContent
 }
 
 impl HtmlLoader {
@@ -22,13 +22,13 @@ impl Loader for HtmlLoader {
 	}
 
 	fn load_buf(&self, _filename: &str, content: Vec<u8>, _chapter: usize) -> Result<Box<dyn Book>> {
-		let lines = html_lines(content)?;
-		Ok(Box::new(HtmlBook { lines }))
+		let content = html_content(content)?;
+		Ok(Box::new(HtmlBook { content }))
 	}
 }
 
 impl Book for HtmlBook {
-	fn lines(&self) -> &Vec<String> {
-		&self.lines
+	fn lines(&self) -> &Vec<Line> {
+		&self.content.lines
 	}
 }
\ No newline at end of file
