@@ -1,14 +1,14 @@
 use anyhow::{anyhow, Result};
 use cursive::{Printer, Vec2, View, XY};
-use cursive::event::{Event, EventResult, Key};
+use cursive::event::{Event, EventResult, Key, MouseButton, MouseEvent};
 use cursive::theme::{ColorStyle, PaletteColor};
-use regex::{Match, Regex};
+use regex::Regex;
 
 use crate::{ContainerManager, ReadingInfo};
-use crate::book::Book;
-use crate::common::{byte_index_for_char, char_index_for_byte};
+use crate::book::{Book, Line};
+use crate::common::char_width;
 use crate::container::Container;
-use crate::controller::{ReverseInfo, update_status_callback};
+use crate::controller::update_status_callback;
 use crate::view::han::Han;
 use crate::view::xi::Xi;
 
@@ -17,15 +17,42 @@ mod xi;
 
 const TRACE_SIZE: usize = 100;
 
-pub struct NextPageInfo {
-	line: usize,
-	position: usize,
+pub enum HighlightMode {
+	Search,
+	Link(usize),
 }
 
-struct TraceInfo {
-	chapter: usize,
-	line: usize,
-	position: usize,
+impl Clone for HighlightMode {
+	fn clone(&self) -> Self {
+		match self {
+			HighlightMode::Search => HighlightMode::Search,
+			HighlightMode::Link(link_index) => HighlightMode::Link(*link_index),
+		}
+	}
+}
+
+pub struct HighlightInfo {
+	pub line: usize,
+	pub start: usize,
+	pub end: usize,
+	pub mode: HighlightMode,
+}
+
+pub struct Position {
+	pub line: usize,
+	pub position: usize,
+}
+
+impl Position {
+	pub fn new(line: usize, position: usize) -> Self {
+		Position { line, position }
+	}
+}
+
+pub struct TraceInfo {
+	pub chapter: usize,
+	pub line: usize,
+	pub position: usize,
 }
 
 pub struct ReadingView {
@@ -40,27 +67,78 @@ pub struct ReadingView {
 	render_context: RenderContext,
 }
 
-pub struct ReverseChar(char, Vec2);
+pub(crate) enum DrawCharMode {
+	Plain,
+	Search,
+	SearchOnLink {
+		line: usize,
+		link_index: usize,
+	},
+	HighlightLink {
+		line: usize,
+		link_index: usize,
+	},
+	Link {
+		line: usize,
+		link_index: usize,
+	},
+}
+
+impl Clone for DrawCharMode {
+	fn clone(&self) -> Self {
+		match self {
+			DrawCharMode::Plain => DrawCharMode::Plain,
+			DrawCharMode::Search => DrawCharMode::Search,
+			DrawCharMode::Link { line, link_index } => DrawCharMode::Link { line: *line, link_index: *link_index },
+			DrawCharMode::HighlightLink { line, link_index } => DrawCharMode::HighlightLink { line: *line, link_index: *link_index },
+			DrawCharMode::SearchOnLink { line, link_index } => DrawCharMode::SearchOnLink { line: *line, link_index: *link_index },
+		}
+	}
+}
+
+pub(crate) struct DrawChar {
+	char: char,
+	mode: DrawCharMode,
+}
+
+impl DrawChar {
+	pub fn new(char: char, mode: DrawCharMode) -> Self {
+		DrawChar { char, mode }
+	}
+	pub fn space() -> Self {
+		DrawChar { char: ' ', mode: DrawCharMode::Plain }
+	}
+}
+
+impl Clone for DrawChar {
+	fn clone(&self) -> Self {
+		DrawChar { char: self.char, mode: self.mode.clone() }
+	}
+}
 
 pub struct RenderContext {
 	width: usize,
 	height: usize,
-	reverse_chars: Vec<ReverseChar>,
-	print_lines: Vec<String>,
-	reverse_color: ColorStyle,
+	print_lines: Vec<Vec<DrawChar>>,
+	search_color: ColorStyle,
+	link_color: ColorStyle,
+	highlight_link_color: ColorStyle,
 	color: ColorStyle,
 	leading_space: usize,
-	next: Option<NextPageInfo>,
+	next: Option<Position>,
 }
 
 impl RenderContext {
 	fn build(book: &Box<dyn Book>) -> Self {
+		let link_color = ColorStyle::new(ColorStyle::secondary().front, PaletteColor::Background);
+		let highlight_link_color = ColorStyle::new(ColorStyle::secondary().front, ColorStyle::highlight().back);
 		RenderContext {
 			width: 0,
 			height: 0,
-			print_lines: vec!["".to_string()],
-			reverse_chars: vec![],
-			reverse_color: ColorStyle::highlight(),
+			print_lines: vec![],
+			search_color: ColorStyle::highlight(),
+			link_color,
+			highlight_link_color,
 			color: ColorStyle::new(PaletteColor::Primary, PaletteColor::Background),
 			leading_space: book.leading_space(),
 			next: None,
@@ -70,12 +148,37 @@ impl RenderContext {
 
 pub(crate) trait Render {
 	fn resized(&mut self, _context: &mut RenderContext) {}
-	fn redraw(&mut self, lines: &Vec<String>, reading: &ReadingInfo, context: &mut RenderContext);
-	fn prev(&mut self, lines: &Vec<String>, reading: &mut ReadingInfo, context: &mut RenderContext);
-	fn next_line(&mut self, lines: &Vec<String>, reading: &mut ReadingInfo, context: &mut RenderContext);
-	fn prev_line(&mut self, lines: &Vec<String>, reading: &mut ReadingInfo, context: &mut RenderContext);
-	// move to reverse line if not displayed in current view
-	fn setup_reverse(&mut self, lines: &Vec<String>, reading: &mut ReadingInfo, context: &mut RenderContext);
+	fn redraw(&mut self, lines: &Vec<Line>, reading: &ReadingInfo, context: &mut RenderContext);
+	fn prev(&mut self, lines: &Vec<Line>, reading: &mut ReadingInfo, context: &mut RenderContext);
+	fn next_line(&mut self, lines: &Vec<Line>, reading: &mut ReadingInfo, context: &mut RenderContext);
+	fn prev_line(&mut self, lines: &Vec<Line>, reading: &mut ReadingInfo, context: &mut RenderContext);
+	// move to highlight line if not displayed in current view
+	fn setup_highlight(&mut self, lines: &Vec<Line>, reading: &mut ReadingInfo, context: &mut RenderContext);
+	fn setup_draw_char(&mut self, char: char, line: usize, position: usize, lines: &Vec<Line>, reading: &ReadingInfo) -> DrawChar {
+		let mut mode = match &reading.highlight {
+			Some(highlight) => if highlight.line == line && highlight.start <= position && highlight.end > position {
+				match highlight.mode {
+					HighlightMode::Search => DrawCharMode::Search,
+					HighlightMode::Link(link_index) => DrawCharMode::HighlightLink { line, link_index },
+				}
+			} else {
+				DrawCharMode::Plain
+			},
+			None => DrawCharMode::Plain,
+		};
+		let text = &lines[line];
+		for (link_index, link) in text.link_iter().enumerate() {
+			if link.range.start <= position && link.range.end > position {
+				match mode {
+					DrawCharMode::Plain => mode = DrawCharMode::Link { line, link_index },
+					DrawCharMode::Search => mode = DrawCharMode::SearchOnLink { line, link_index },
+					_ => {}
+				};
+				break;
+			}
+		}
+		DrawChar { char, mode }
+	}
 }
 
 fn load_render(render_type: &String) -> Box<dyn Render> {
@@ -86,44 +189,26 @@ fn load_render(render_type: &String) -> Box<dyn Render> {
 	}
 }
 
-fn is_reverse_displayed(reading: &ReadingInfo, context: &RenderContext) -> bool {
-	let reverse = match &reading.reverse {
-		Some(r) => r,
-		None => return true,
-	};
-	let revers_line = reverse.line;
-	let revers_start = reverse.start;
-	let reading_line = reading.line;
-	let reading_position = reading.position;
-	if (revers_line == reading_line && revers_start >= reading_position) || (revers_line > reading_line) {
-		match &context.next {
-			Some(next) => if (revers_line == next.line && revers_start < next.position) || (revers_line < next.line) {
-				return true;
-			}
-			None => return true,
-		}
-	}
-	return false;
-}
-
 impl View for ReadingView {
 	fn draw(&self, printer: &Printer) {
 		let context = &self.render_context;
-		printer.with_color(context.color, |printer| {
-			let mut xy = XY { x: 0, y: 0 };
-			for line in &context.print_lines {
-				printer.print(xy, line.as_str());
-				xy.y += 1;
+		let mut xy = XY { x: 0, y: 0 };
+		let mut tmp = [0u8; 4];
+		for line in &context.print_lines {
+			for dc in line {
+				let color = match dc.mode {
+					DrawCharMode::Plain => context.color,
+					DrawCharMode::Search | DrawCharMode::SearchOnLink { .. } => context.search_color,
+					DrawCharMode::Link { .. } => context.link_color,
+					DrawCharMode::HighlightLink { .. } => context.highlight_link_color,
+				};
+				printer.with_color(color, |printer| {
+					printer.print(xy, dc.char.encode_utf8(&mut tmp));
+				});
+				xy.x += char_width(dc.char);
 			}
-		});
-		let reverse_chars = &context.reverse_chars;
-		if reverse_chars.len() > 0 {
-			let mut tmp = [0u8; 4];
-			printer.with_color(context.reverse_color, |printer| {
-				for ReverseChar(ch, xy) in reverse_chars {
-					printer.print(xy, ch.encode_utf8(&mut tmp));
-				}
-			});
+			xy.x = 0;
+			xy.y += 1;
 		}
 	}
 
@@ -277,16 +362,16 @@ impl ReadingView {
 				self.push_trace(true);
 			}
 			Event::Char('n') => {
-				let (line, position) = match &self.reading.reverse {
-					Some(reverse) => (reverse.line, reverse.end),
-					None => (self.reading.line, self.reading.position),
+				let (line, position) = match &self.reading.highlight {
+					Some(HighlightInfo { mode: HighlightMode::Search, line, end, .. }) => (*line, *end),
+					None | Some(HighlightInfo { mode: HighlightMode::Link(..), .. }) => (self.reading.line, self.reading.position),
 				};
 				self.search_next(line, position)?;
 			}
 			Event::Char('N') => {
-				let (line, position) = match &self.reading.reverse {
-					Some(reverse) => (reverse.line, reverse.start),
-					None => (self.reading.line, self.reading.position),
+				let (line, position) = match &self.reading.highlight {
+					Some(HighlightInfo { mode: HighlightMode::Search, line, start, .. }) => (*line, *start),
+					None | Some(HighlightInfo { mode: HighlightMode::Link(..), .. }) => (self.reading.line, self.reading.position),
 				};
 				self.search_prev(line, position)?;
 			}
@@ -300,6 +385,29 @@ impl ReadingView {
 			}
 			Event::Key(Key::Right) => self.goto_trace(false)?,
 			Event::Key(Key::Left) => self.goto_trace(true)?,
+
+			Event::Key(Key::Tab) => self.switch_link_next()?,
+			Event::Shift(Key::Tab) => self.switch_link_prev()?,
+			Event::Key(Key::Enter) => {
+				match self.reading.highlight {
+					Some(HighlightInfo { mode: HighlightMode::Search, line, start, end }) => {
+						let text = &self.book.lines()[line];
+						for (link_index, link) in text.link_iter().enumerate() {
+							let range = &link.range;
+							if range.start <= start && range.end >= end {
+								self.goto_link(line, link_index)?;
+								break;
+							}
+						}
+					}
+					Some(HighlightInfo { mode: HighlightMode::Link(link_index), line, .. }) => {
+						self.goto_link(line, link_index)?;
+					}
+					None => {}
+				}
+			}
+			Event::Mouse { event: MouseEvent::Press(MouseButton::Left), position, .. } =>
+				self.left_click(position)?,
 			_ => return Ok(false),
 		};
 		Ok(true)
@@ -323,15 +431,8 @@ impl ReadingView {
 					if book_index >= book_count {
 						Ok(false)
 					} else {
-						let reading = ReadingInfo {
-							filename: self.reading.filename.clone(),
-							inner_book: book_index,
-							chapter: 0,
-							line: 0,
-							position: 0,
-							ts: 0,
-							reverse: None,
-						};
+						let reading = ReadingInfo::new(&self.reading.filename)
+							.with_inner_book(book_index);
 						self.do_switch_book(reading)?;
 						Ok(true)
 					}
@@ -356,19 +457,13 @@ impl ReadingView {
 				Ok(true)
 			} else {
 				if reading.inner_book > 0 {
-					let mut new_reading = ReadingInfo {
-						filename: reading.filename.clone(),
-						inner_book: reading.inner_book - 1,
-						chapter: usize::MAX,
-						line: 0,
-						position: 0,
-						ts: 0,
-						reverse: None,
-					};
+					let mut new_reading = ReadingInfo::new(&reading.filename)
+						.with_inner_book(reading.inner_book - 1)
+						.with_last_chapter();
 					self.book = load_book(&self.container_manager, &mut self.container, &mut new_reading)?;
 					new_reading.chapter = self.book.current_chapter();
 					new_reading.line = self.book.lines().len();
-					new_reading.position = self.book.lines()[new_reading.line - 1].chars().count();
+					new_reading.position = self.book.lines()[new_reading.line - 1].len();
 					self.reading = new_reading;
 					self.prev_page()?;
 					self.trace.clear();
@@ -386,20 +481,13 @@ impl ReadingView {
 		}
 	}
 
-	fn build_reverse(text: &str, line: usize, m: Match) -> Option<ReverseInfo> {
-		Some(ReverseInfo {
-			line,
-			start: char_index_for_byte(text, m.start()).unwrap(),
-			end: char_index_for_byte(text, m.end()).unwrap(),
-		})
-	}
-
 	pub(crate) fn switch_chapter(&mut self, chapter: usize) -> String {
 		match self.switch_chapter_internal(chapter) {
 			Ok(_) => self.status_msg(),
 			Err(e) => e.to_string(),
 		}
 	}
+
 	fn switch_chapter_internal(&mut self, chapter: usize) -> Result<bool> {
 		if chapter < self.book.chapter_count() {
 			self.book.set_chapter(chapter)?;
@@ -422,20 +510,18 @@ impl ReadingView {
 		let book = &self.book;
 		let lines = book.lines();
 		let regex = Regex::new(search_text.as_str())?;
-		let mut position = byte_index_for_char(&lines[start_line], start_position).unwrap();
+		let mut position = start_position;
 		for idx in start_line..lines.len() {
 			let line = &lines[idx];
-			match regex.find_at(line, position) {
-				Some(m) => {
-					self.reading.reverse = Self::build_reverse(line, idx, m);
-					if !is_reverse_displayed(&self.reading, &self.render_context) {
-						self.render.setup_reverse(lines, &mut self.reading, &mut self.render_context);
-					}
-					self.render.redraw(self.book.lines(), &self.reading, &mut self.render_context);
-					self.push_trace(false);
-					return Ok(());
-				}
-				None => (),
+			if let Some(range) = line.search_pattern(&regex, Some(position), None, false) {
+				self.reading.highlight = Some(HighlightInfo {
+					line: idx,
+					start: range.start,
+					end: range.end,
+					mode: HighlightMode::Search,
+				});
+				self.highlight_setup();
+				return Ok(());
 			}
 			position = 0;
 		}
@@ -450,32 +536,30 @@ impl ReadingView {
 		let lines = self.book.lines();
 		let regex = Regex::new(search_text.as_str())?;
 		for idx in (0..=start_line).rev() {
-			let text = if idx == start_line {
+			let range = if idx == start_line {
 				if start_position == 0 {
 					continue;
 				} else {
-					let text = &lines[idx];
-					let byte_index = byte_index_for_char(text, start_position).unwrap();
-					&text[0..byte_index]
+					lines[idx].search_pattern(&regex, None, Some(start_position), true)
 				}
 			} else {
-				&lines[idx]
+				lines[idx].search_pattern(&regex, None, None, true)
 			};
-			match regex.find_iter(text).last() {
-				Some(m) => {
-					self.reading.reverse = Self::build_reverse(text, idx, m);
-					self.render.setup_reverse(lines, &mut self.reading, &mut self.render_context);
-					self.render.redraw(self.book.lines(), &self.reading, &mut self.render_context);
-					self.push_trace(false);
-					return Ok(());
-				}
-				None => (),
+			if let Some(range) = range {
+				self.reading.highlight = Some(HighlightInfo {
+					line: idx,
+					start: range.start,
+					end: range.end,
+					mode: HighlightMode::Search,
+				});
+				self.highlight_setup();
+				return Ok(());
 			}
 		}
 		Ok(())
 	}
 
-	fn push_trace(&mut self, clear_reverse: bool) {
+	fn push_trace(&mut self, clear_highlight: bool) {
 		let reading = &self.reading;
 		let trace = &mut self.trace;
 		let last = &trace[self.current_trace];
@@ -489,8 +573,8 @@ impl ReadingView {
 		} else {
 			self.current_trace += 1;
 		}
-		if clear_reverse {
-			self.reading.reverse = None;
+		if clear_highlight {
+			self.reading.highlight = None;
 		}
 	}
 	fn goto_trace(&mut self, backward: bool) -> Result<()> {
@@ -517,9 +601,156 @@ impl ReadingView {
 			self.book.set_chapter(reading.chapter)?;
 		}
 		self.render.redraw(self.book.lines(), reading, &mut self.render_context);
-		self.reading.reverse = None;
+		self.reading.highlight = None;
 		Ok(())
 	}
+
+	fn switch_link_prev(&mut self) -> Result<()> {
+		let reading = &mut self.reading;
+		let (mut line, mut position) = match &reading.highlight {
+			Some(HighlightInfo { mode: HighlightMode::Link(..), line, start, .. }) => (*line, *start),
+			None | Some(HighlightInfo { mode: HighlightMode::Search, .. }) => (reading.line, reading.position)
+		};
+		let lines = self.book.lines();
+		let mut text = &lines[line];
+		'outer: loop {
+			for (link_index, link) in text.link_iter().rev().enumerate() {
+				if link.range.end <= position {
+					reading.highlight = Some(HighlightInfo {
+						line,
+						start: link.range.start,
+						end: link.range.end,
+						mode: HighlightMode::Link(link_index),
+					});
+					break 'outer;
+				}
+			}
+			if line == 0 {
+				break;
+			}
+			line -= 1;
+			text = &lines[line];
+			position = text.len();
+		}
+		self.highlight_setup();
+		Ok(())
+	}
+
+	fn switch_link_next(&mut self) -> Result<()> {
+		let reading = &mut self.reading;
+		let (line, mut position) = match &reading.highlight {
+			Some(HighlightInfo { mode: HighlightMode::Link(..), line, end, .. }) => (*line, *end),
+			None | Some(HighlightInfo { mode: HighlightMode::Search, .. }) => (reading.line, reading.position),
+		};
+		let lines = self.book.lines();
+		'outer: for index in line..lines.len() {
+			let text = &lines[index];
+			for (link_index, link) in text.link_iter().enumerate() {
+				if link.range.start >= position {
+					reading.highlight = Some(HighlightInfo {
+						line: index,
+						start: link.range.start,
+						end: link.range.end,
+						mode: HighlightMode::Link(link_index),
+					});
+					break 'outer;
+				}
+			}
+			position = 0;
+		}
+		self.highlight_setup();
+		Ok(())
+	}
+
+	fn left_click(&mut self, position: Vec2) -> Result<()> {
+		let print_lines = &self.render_context.print_lines;
+		if let Some(print_line) = print_lines.get(position.y) {
+			let mut x = 0;
+			for index in 0..print_line.len() {
+				if x >= position.x {
+					let dc = if x == position.x {
+						&print_line[index]
+					} else {
+						// for cjk char draw at position x, but click at x+1
+						&print_line[index - 1]
+					};
+					match dc.mode {
+						DrawCharMode::Link { line, link_index, .. }
+						| DrawCharMode::HighlightLink { line, link_index, .. }
+						| DrawCharMode::SearchOnLink { line, link_index } =>
+							self.goto_link(line, link_index)?,
+						DrawCharMode::Search | DrawCharMode::Plain => {}
+					}
+					break;
+				}
+				let wc = char_width(print_line[index].char);
+				x += wc;
+			}
+		}
+		// let option = self.render_context.special_char_map
+		// 	.get(&position)
+		// 	.and_then(|mode| {
+		// 		match mode {
+		// 			DrawCharMode::Link { line, link_index, .. }
+		// 			| DrawCharMode::HighlightLink { line, link_index, .. }
+		// 			| DrawCharMode::SearchOnLink { line, link_index } => {
+		// 				Some((line, link_index))
+		// 			}
+		// 			DrawCharMode::Search | DrawCharMode::Plain => None
+		// 		}
+		// 	});
+		// if let Some((line, link_index)) = option {
+		// 	self.goto_link(*line, *link_index)?;
+		// }
+		Ok(())
+	}
+
+	fn goto_link(&mut self, line: usize, link_index: usize) -> Result<()> {
+		if let Some(pos) = self.book.link_position(line, link_index) {
+			if pos.chapter != self.book.current_chapter() {
+				self.book.set_chapter(pos.chapter)?;
+			}
+			self.reading.chapter = pos.chapter;
+			self.reading.line = pos.line;
+			self.reading.position = pos.position;
+			if pos.position == 0 {
+				self.render.redraw(self.book.lines(), &self.reading, &mut self.render_context);
+			} else {
+				self.render.prev_line(self.book.lines(), &mut self.reading, &mut self.render_context);
+			}
+			self.push_trace(true);
+		}
+		Ok(())
+	}
+
+	fn highlight_setup(&mut self) {
+		let in_current_screen = match &self.reading.highlight {
+			Some(highlight) => {
+				let highlight_line = highlight.line;
+				let highlight_start = highlight.start;
+				let reading_line = self.reading.line;
+				let reading_position = self.reading.position;
+				if (highlight_line == reading_line && highlight_start >= reading_position) || (highlight_line > reading_line) {
+					match &self.render_context.next {
+						Some(next) => if (highlight_line == next.line && highlight_start < next.position) || (highlight_line < next.line) {
+							true
+						} else {
+							false
+						}
+						None => true,
+					}
+				} else {
+					false
+				}
+			}
+			None => true,
+		};
+		if !in_current_screen {
+			self.render.setup_highlight(self.book.lines(), &mut self.reading, &mut self.render_context);
+			self.push_trace(false);
+		}
+		self.render.redraw(self.book.lines(), &self.reading, &mut self.render_context);
+	}
 }
 
 fn load_container(container_manager: &ContainerManager, reading: &ReadingInfo) -> Result<Box<dyn Container>> {
@@ -537,7 +768,7 @@ fn load_book(container_manager: &ContainerManager, container: &mut Box<dyn Conta
 	if reading.line >= lines.len() {
 		reading.line = lines.len() - 1;
 	}
-	let chars = lines[reading.line].chars().count();
+	let chars = lines[reading.line].len();
 	if reading.position >= chars {
 		reading.position = 0;
 	}
